#! /usr/bin/perl

##############################################################################
#                                                                            #
# bcm.pl - Perl-Run-Utility-Bibliothek fuer die Linux-BayCom-Mailbox         #
#                                                                            #
# Version: 1.1 (17.10.96)                                                    #
#                                                                            #
# (C) by DG4MMI 1996                                                         #
#                                                                            #
##############################################################################

# GetRange: Zerlegt eine Bereichsangabe in die zwei einzelnen Zahlen
#
# Eingabe: - String, der eine Bereichsangabe enthalten soll
#
# Ausgabe: - Die zwei einzelnen Zahlen
#          - zwei gleiche negative Zahlen, wenn ein Bereich
#            wie "-100" angegeben wurde
#          - die zweite Zahl mit 999999, wenn ein Bereich
#            wie "100-" angegeben wurde
#          - Leerstring, wenn: * der String ein unzulaessiges Zeichen enthaelt
#                              * die erste Zahl groesser ist als die zweite
#                              
sub GetRange {
	local($RangeBegin, $RangeEnd, $Dummy);
	($RangeBegin, $RangeEnd, $Dummy) = split(/-/, $_[0]);
	if($RangeBegin eq "") {
		$RangeEnd = -1 * $RangeEnd;
		$RangeBegin = $RangeEnd;
	}
	if(($RangeEnd eq "") && ($Dummy eq "")) {
		$RangeEnd = 999999;
	}
	if(($Dummy ne "") || ($RangeBegin == 0) || ($RangeEnd == 0) ||
	   ($RangeBegin > $RangeEnd)) { return; }
	return($RangeBegin, $RangeEnd);
}

# GetParam: Sucht aus einer bestimmten Datei einen Parameter und
#           gibt den zugehoerigen Wert aus
#
# Eingabe: - Gewuenschter Parameter (Gross-/Kleinschreibung egal)
#
# Ausgabe: - Wert des Parameters
#          - Leerstring, wenn: * kein Parameter uebergeben wurde
#                              * die angegebene Datei nicht gefunden wurde
#                              * der Parameter in der Datei nicht existiert
#
sub GetParam {
	local($FileName) = @_[0];
	local($ParName) = @_[1];
	if(($FileName eq "") || ($ParName eq "")) { return(""); }
	if(! open(InitFile, $FileName)) { return(""); }
	$ParName =~ y/A-Z/a-z/;
	while(<InitFile>) {
		chop($_);
		if($_ =~ /^$ParName\b/) {
		close(InitFile);
		local($Setting) = substr($_, index($_, " ") + 1);
		$Setting =~ s/^ +//;
		$Setting =~ s/ +$//;
		return($Setting);
		}	
	}
	return("");
}

# FindBoard: Gibt zu einem Board das Main- und das Sub-Board aus
#
# Eingabe: - Boardname (Gross-/Kleinschreibung egal)
#
# Ausgabe: - Voller Boardname in der Form "MainBoard/SubBoard"
#          - Leerstring, wenn: * kein Parameter uebergeben wurde
#                              * bulletin.bcm nicht gefunden wurde
#                              * das Board nicht existiert
#
sub FindBoard {
	local($ReadBoard);
	local($BoardName) = @_[0];
	if($BoardName eq "") { return(""); }
	if(! open(BoardFile, "/bcm/bulletin.bcm")) { return("xxx"); }
	$BoardName =~ y/A-Z/a-z/;
	while(<BoardFile>) {
		$ReadBoard = substr($_, 0, 10);
		$ReadBoard =~ s/ +//g;
		if($_ =~ /^ /) {
			$SubBoard = "/$ReadBoard";
		} else {
			$MainBoard = $ReadBoard;
			$SubBoard = "";
		}
		if($ReadBoard eq $BoardName) {
			return("$MainBoard$SubBoard");
		}
	}
	return("");
}

# BoardPath: Gibt den vollstaendigen Pfad zu einem Bulletin- oder
#            User-Board aus
#
# Eingabe: - BoardName (Gross-/Kleinschreibung egal)
#
# Ausgabe: - Vollstaendiger Pfad im Datei-System zum angegebenen Board
#          - Leerstring, wenn: * kein Parameter uebergeben wurde
#                              * bulletin.bcm nicht gefunden wurde
#                              * init.bcm nicht gefunden wurde
#                              * das Board nicht existiert
#
sub BoardPath {
	local($BoardName) = @_[0]; # !! Wird auch von FindBoard() verwendet !!
	if($BoardName eq "") { return(""); }
	$BoardName =~ y/A-Z/a-z/;
	if(opendir(TestUser, &GetParam("/bcm/init.bcm", "userpath")."/".$BoardName)) {
		closedir(TestUser);
		return(&GetParam("/bcm/init.bcm", "userpath")."/".$BoardName);
	} else {
		if(&FindBoard("$BoardName") ne "") {
			return(&GetParam("/bcm/init.bcm", "infopath")."/".
		        &FindBoard("$BoardName"));
		} else { 
			return(""); 
		}
	}
}

# MailFileName: Gibt den Datei-Namen einer Mail aus
#
# Eingabe: - BoardName (Gross-/Kleinschreibung egal)
#          - Mail-Nummer
#
# Ausgabe: - Datei-Name der Mail (ohne Pfad)
#          - Leerstring, wenn: * kein oder nur ein Parameter uebergeben wurde
#                              * bulletin.bcm nicht gefunden wurde
#                              * init.bcm nicht gefunden wurde
#                              * das Board nicht existiert
#                              * eine Mail mit dieser Nummer nicht existiert
#
sub MailFileName {
	if((! open(ListFile, &BoardPath($_[0])."/list.bcm")) || ($_[1] < 1)) {
		return;
	}
	for($MailCount = 0;$MailCount < $_[1];$MailCount++) {
		$FileName = substr(<ListFile>, 0, 7);
	}
	close(ListFile);
	$FileName =~ y/A-Z/a-z/;
	return($FileName);
}

# BCMnum: Wandelt eine im BCM-Format komprimierte Zahl in eine echte Zahl um
#
# Eingabe: - die Zahl im BCM-Format
#
# Ausgabe: - die Zahl im (normalen) Zehner-System
#
sub BCMnum {
	local($Count, $RealNum);

	for($Count = 0;$Count < length($_[0]);$Count++) {
		$RealNum += (ord(substr($_[0], $Count, 1)) - 33) *
		            128 ** (length($_[0]) - $Count - 1);
	}

	return($RealNum);
}

# Dec2Hex: Wandelt eine Dezimal-Zahl in einen Hexadezimal-Zahl um
#
# Eingabe: - die Zahl im Dezimal-Format
#          - die Anzahl an Stellen, auf die die Hexadezimal-Zahl mit
#            fuehrenden Nullen aufgefuellt werden soll
#
# Ausgabe: - die Zahl im Hexadezimal-Format (aber ohne "0x")
#
sub Dec2Hex {
	local($Count, $DecNum, $HexNum, $HexChar) = 0;

	$DecNum = $_[0];

	while(($DecNum / (16 ** $Count)) >= 16) { $Count++; }

	for(;$Count >= 0;$Count--) {
		$HexChar = int($DecNum / (16 ** $Count));
		$DecNum -= $HexChar * (16 ** $Count);
		if($HexChar < 10) {
			$HexNum .= chr(48 + $HexChar);
		} else {
			$HexNum .= chr(87 + $HexChar);
		}
	}

	$HexNum = "0" x ($_[1] - length($HexNum)) . $HexNum;

	return($HexNum);
}

# RemovePath: Schneidet den Pfad von einer Datei-Angabe ab
#
# Ein-/Ausgabe: Es wird direkt die angegebene Variable umgewandelt
#               Die Angabe von Backslashes statt Slashes ist moeglich
#
sub RemovePath {
	$_[0] =~ s/\\/\//g;
	$_[0] = substr($_[0], rindex($_[0], "/") + 1);
}

# CutName: Zerlegt einen DateiNamen in Pfad, Name und Extension
#
# Eingbae: Der Name einer Datei mit Pfad
#
# Ausgabe: - Pfad (mit abschliessendem Slash)
#          - Dateiname (ohne Extension)
#          - Dateiextension (mit fuehrendem Punkt)
#
sub CutName {
	local($Path, $Name, $Ext);

	$Path=substr($_[0], 0, rindex($_[0], "/") + 1);
	$Name=substr($_[0], rindex($_[0], "/") + 1);
	if($Name =~ /\./) {
		$Ext=substr($Name, index($Name, "."));
		$Name=substr($Name, 0, index($Name, "."));
	}

	return($Path, $Name, $Ext);
}



1;
