/***************************************************************

  BayCom(R)        Packet-Radio fuer IBM PC

  OpenBayCom-Mailbox

  -----------------
  Diverse Utilities
  -----------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980115 OE3DZW added ad_timezone()
//                added DOS-timezone instead of utcoffset
//20000130 OE3DZW time zone was not correct under Linux when TZ was set

#include "baycom.h"

char     globalpath [80];

/*---------------------------------------------------------------------------*/

#ifdef __DOS16__
void fixpath (char *argv0)
//*************************************************************************
//
//  Stellt den Pfad fest, aus dem das Programm gestartet worden ist
//  und legt ihn in globalpath ab
//
//*************************************************************************
{ unsigned int i = 0;
  char psave[80];

  strcpy (psave, argv0);
  i = strlen (psave);
  while (i && (psave[i] != '/' && psave[i] != '\\')) i--;
  for ( ; i + 1; i--) globalpath[i] = psave[i];
  killbackslash (globalpath);
}
#else
void fixpath (char *)
{ char buf[80];
  char *s = getenv ("BCMHOME");
  if ( ! s) s = getcwd (buf, 80);
  strcpy (globalpath, s);
  killbackslash (globalpath);
  trace (report, "working dir", "'%s'", globalpath);
  if ( ! *globalpath || globalpath[strlen (globalpath) - 1] != '/') strcat (globalpath, "/");
}
#endif

/*---------------------------------------------------------------------------*/

struct tm *ad_comtime (time_t timeu)
{
  return localtime (&timeu);
}

/*---------------------------------------------------------------------------*/

time_t ad_mktime (struct tm *tt)
{
  return mktime (tt);
}

/*---------------------------------------------------------------------------*/

time_t loctime (void)
//*************************************************************************
//
//  Gibt Zeit im ANSI-Format zurueck (Sekunden seit 1970-01-01).
//  Haeufige Aufrufe werden nur dann zur Software durchgereicht,
//  wenn die Sekunde tatsaechlich umgeschlagen hat.
//
//  Hat sich die Zeit nach unten veraendert, so wird die Differenz
//  zur bisherigen Zeit gebildet und die tatsaechliche Zeit sukzessive
//  angenaehert.
//
//*************************************************************************
{
  static long lasttic = 0;
  static time_t time_correct = 0;
  static time_t last_decrement = 0;
  static time_t last_time = time (NULL);
  time_t newvalue;

  // mittels schnellen Ticker-Vergleich feststellen, ob sich die
  // Zeit veraendert hat.
  if (lasttic < (systic + (1000 / MS_PER_TIC) - 1) || lasttic > systic)
  {
    lasttic = systic;
    newvalue = time (NULL);
    // ist die Uhr zurueckgelaufen?
    if (newvalue < last_time)
    {
      // Differenz merken
      time_correct = last_time - newvalue;
      last_decrement = newvalue;
    }
    // wurde die Uhr vorgestellt aber es ist noch eine Differenz aktiv?
    if (newvalue > (last_time + 1))
    { if (time_correct > (newvalue - last_time + 1)) time_correct -= (newvalue - last_time);
      else time_correct = 0;
      last_decrement = newvalue;
    }
    // ist eine Differenz aktiv?
    if (time_correct > 0  && newvalue > (last_decrement + 1))
    {
      // alle 2 Sekunden die Differenz um eine Sekunde verringern
      time_correct--;
      last_decrement = newvalue;
    }
    last_time = newvalue;
  }
  return last_time + time_correct;
}

/*---------------------------------------------------------------------------*/

#ifndef __LINUX__
time_t isdst (void)
//---------------------------------------------------------------------------
// returns daylight-saving in seconds depending on summertime
// used in WIN32 and DOS16
//---------------------------------------------------------------------------
{
  time_t ti = loctime ();
  struct tm *tt;
  tt = ad_comtime (ti);
  if (tt->tm_mon > 2 && tt->tm_mon < 9) return ((time_t) m.summertime * 3600);
  else
  { if (   tt->tm_mon == 2  // March
        && tt->tm_mday > 23
        && (31 - tt->tm_mday) <= (7 - tt->tm_wday)
        && (tt->tm_wday || tt->tm_hour >= 2)) return ((time_t) m.summertime * 3600);
    if (   tt->tm_mon == 9  //October
        && (   tt->tm_mday < 22
            || (30 - tt->tm_mday) > (7 - tt->tm_wday)
            || ( ! tt->tm_wday && tt->tm_hour <= 3))) return ((time_t) m.summertime * 3600);
  }
  return (time_t) 0L; // winter time
}
#endif

/*---------------------------------------------------------------------------*/

time_t ad_timezone (void)
//---------------------------------------------------------------------------
// returns offset between utc and local time in seconds
// ad_timezone() depends of setting the environment variable TZ
//---------------------------------------------------------------------------
{
#ifdef __FLAT__
#ifdef __LINUX__
  struct tm *tt;
  time_t ut = loctime (); //bcm local time
  tt = ad_comtime (ut);
  return tt->tm_gmtoff;
#else //_WIN32
  return (isdst() - _timezone);
//  _daylight   Nonzero if daylight saving time (DST) zone is specified in TZ; otherwise, 0. Default value is 1.
//  _timezone   Difference in seconds between coordinated universal time and local time. Default value is 28,800.
//  _tzname[0]  Time-zone name derived from TZ environment variable.
//  _tzname[1]  DST zone name derived from TZ environment variable. Default value is PDT (Pacific daylight time).
//              If DST zone is omitted from TZ, _tzname[1] is empty string.
#endif
#else //__DOS16__
  if (m.pcisutc) return (time_t) 0L;
  else return (isdst () + m.stimeoffset * 3600);
#endif
}

/*---------------------------------------------------------------------------*/

time_t ad_time (void)
//make UTC, Referenz for all internal ANSI times of bcm
{
  return loctime () - ad_timezone ();
}

/*---------------------------------------------------------------------------*/

unsigned ad_minutes (void)
{ time_t ti = ad_time ();
  struct tm *tt;
  tt = ad_comtime (ti);
  return 60 * tt->tm_hour + tt->tm_min;
}

/*---------------------------------------------------------------------------*/
