/***************************************************************

  BayCom(R)        Packet-Radio fuer IBM PC

  OpenBayCom-Mailbox

  Routinen fuer Benutzeroberflaeche


  -----------------------------------------------
  Definitionen und Prototypen fuer Taskverwaltung
  -----------------------------------------------


  Copyright (c) 12/91 Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/
//19980311 DG9MHZ changes for Linux Kernel AX25
//19980615 hrx    reimplemented downlink (added to task_t)

#ifndef _TASKING_H
#define _TASKING_H

#define NOTASK    (-1)

// Flags fuer laufenden Prozess (wird in t->processkind hinterlegt)
// Die Flags koennen beliebig kombiniert werden
//
#define P_MAIL 1      // hat einen Mailbox-Kontrollblock
#define P_WIND 2      // hat einen Editor-Kontrollblock
#define P_KEYB 4      // benutzt Tastatureingaben
#define P_BACK 8      // laeuft im Hintergrund (keinerlei Ein- und Ausgabe)
#define P_CLON 16     // DH3MB: Mailbox-Kontrollblock wird in die neue
                      // Task uebernommen
#define P_AX25K  0x100
#define P_TELNET 0x200


typedef unsigned long bitfeld;

typedef enum event_t
{ e_nothing,
  e_ticsfull,
  e_reschedule,
  e_delay,
  e_kbhit
} event_t;

typedef enum io_t
{ io_dummy,
  io_window,
  io_editwin,
  io_tnc,
  io_tty,
  io_file,
  io_socket,
  io_memory,
  io_comp // DH3MB: Online-COMP
} io_t;

#define MONBUFLEN 2048

typedef struct monbuf_t
{ int      tid;
  bitfeld  opt;
  unsigned in;
  unsigned out;
  int      congested;
  char     buf[MONBUFLEN];
} monbuf_t;
/*
typedef struct sock_t
{
  char rxbuf[RXLEN];
  char txbuf[TXLEN];
  char mycall[16];
  int handle;
  int rxc;
  int txc;
  int rxlen;
  int wasput;
  int wasda;
  char dasda;
  time_t lastflush;
}
sock_t;
*/
typedef void (* atkill_t)(void);
typedef void (* atkill2_t)(void *);
#define MAX_ATKILL 4

typedef struct task_t
{ event_t  event;
  int      taskid;
  int      port;
  int      lock;                // >0 avoids re-scheduling
  int      socket;
  int      childpid;
  io_t     input;
  io_t     output;
  monbuf_t *monitor;
  unsigned lastdelay;
  unsigned begintic;
  long     cputics;
  jmp_buf  j;
  unsigned processkind;
  time_t   created;
  time_t   last_input;
  time_t   kill_time;
  time_t   warn_time;
  int      kill_warn;
  unsigned keybuffer[32];
  int      keybufin;
  int      keybufout;
  void     *e;
  void     *b;
  atkill2_t atkill[MAX_ATKILL];
  void     *atkill_p[MAX_ATKILL];
  unsigned atkillcnt;
  char     name[20];
  // the sequence of lastcmd, lastfunc and stack is important for the stack
  // overflow detection verify_stack(), don't change it! (db1ras)
  char     lastcmd[60];        // letztes eingegebenes Kommando
  char     lastfunc[40];       // letzte aufgerufene Funktion
  char     stack[TASKSTACK+4]; // Stack
  char     downlink[10];       // letztes Digirufzeichen
} task_t;

#define _s_report      0
#define _s_replog      1
#define _s_serious     2
#define _s_fatal       3
#define _s_tr_abbruch  4

#ifdef __FLAT__
#define report      _s_report,__LINE__,__FILE__
#define replog      _s_replog,__LINE__,__FILE__
#define serious     _s_serious,__LINE__,__FILE__
#define fatal       _s_fatal,__LINE__,__FILE__
#define tr_abbruch  _s_tr_abbruch,__LINE__,__FILE__
#else
#define report      _s_report,0,""
#define replog      _s_replog,0,""
#define serious     _s_serious,0,""
#define fatal       _s_fatal,0,""
#define tr_abbruch  _s_tr_abbruch,0,""
#endif

extern int newkeywin;
extern int keytaskid;
extern int deskkey;
extern int init_off;
extern task_t *t;       // Zeiger auf aktuellen TCB
extern long systic;

void call_l2 (int off);

//****************************
//       mbtask.cpp
//****************************
int      fork (int typ, int port, void(*stfunc)(char *), char *stpar);
void     atkill (atkill_t func);
void     atkill (atkill2_t func, void *p);
void     rm_atkill (atkill_t func);
void     suicide (void);
int      waitfor (event_t event);
#ifdef __MSDOS__
unsigned waitkey (void);
#endif
unsigned waitalt (void);
int      wdelay (unsigned zeit);
void     maintask (void);
void     initstack (void);
void     setkeyhere (void);
int      iskeytask (void);
int      keywindow (void);
int      gettaskid (void);
char *   gettaskname (int tid);
void     listprocess (long opt);
int      killtask (int tid, int discon_);
int      killport (int port);
int      taskport0 (int port);
unsigned listusers (unsigned putit);
unsigned eingelogt(char *logincall, int job, int count);
#ifdef _TTY
int      taskexists (int tid); //JJ
#endif
int      taskkbhit (void);
int      mbtalk (char *fromcall, char *tocall, char *message);
int      monitor (int tid, bitfeld opt);
int      setkeywindow (int fenster);
void     lastcmd (char *cmd);
void     lastfunc (char *cmd);
void     post_mortem_dump (char *);
void     task_idle (int active);
void     timeout (int minutes);
void     test_killtime (void);
void     t_lock (void);
void     t_unlock (int force);

//****************************
//       mbtrace.cpp
//****************************
void     log_entry (char *logname, char *msg);
void     trace (unsigned mode, int line, char *file, char *programm, char *string, ...);
void     tracewindow (char *name);
void     cmdlog (char *befehl);
void     tracelog (char *message, unsigned mode);
void     pwlog (char *mycall, char *digi, char *msg);

//****************************
//       mballoc.cpp
//****************************
void    *t_malloc (size_t size, char *name);
void    *t_realloc (void *blk, size_t newsize);
void    t_free (void *blk);
void    t_alloclist (char* befbuf); //df3vi: sortieroptionen
void    t_allocinit (void);
void    t_allfree (int task);
void    t_checkfree (void);
int     t_settaskid (void *blk, int tid);
#endif
